// Diafaan SMS Server Scripting Connector skeleton script
// AddReference System.dll
// AddReference System.Web.dll
// AddReference C:\Program Files (x86)\Diafaan SMS Server\HttpServer.dll
//
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Net;
using System.Web;
using System.Text;
using System.IO;
using HttpServer;

namespace DiafaanMessageServer
{
	public class ConnectorScript : IScript
	{
		private int httpServerPort = 8080;
		private int httpServerBacklog = 1000;
		
		private string GetHttpCallbackResponse(NameValueCollection queryList)
		{
			string response = "result=1";
			
//			string action = GetQueryParameter(queryList, "action");			
//			if (action == "message_out") {
//			}
//			if (action == "message_in") {
//			}
//			if (action == "message_log") {
//			}
//			if (action == "message_log_update") {
//			}
//			if (action == "event_log") {
//			}
//			if (action == "validate_user") {
//			}
//			if (action == "accept_connection") {
//			}
			return response;
		}
		
		private IScriptHost host = null;
		public void OnLoad(IScriptHost host)
		{
			this.host = host;
			StartHttpServer();
		}

		public void OnUnload()
		{
			StopHttpServer();
		}
		#region HTTP server functions
		private HttpServer.HttpListener httpListener = null;
		private bool StartHttpServer()
		{
			bool returnValue = false;
			
			try {
				httpListener = HttpServer.HttpListener.Create(IPAddress.Any, httpServerPort);
				httpListener.RequestReceived += OnRequest;
				httpListener.Start(httpServerBacklog);
				PostEventLog(String.Format("HTTP Callback Server started on host {0}, port {1}.", Dns.GetHostName(), httpServerPort), "", EventLog.Information);
				returnValue = true;
			}
			catch (Exception e) {
				PostEventLog(e.Message, e.ToString(), EventLog.Error);					
			}
			return returnValue;
		}
		private void StopHttpServer()
		{
			try {
				httpListener.Stop();
				PostEventLog(String.Format("HTTP Callback Server stopped."), "", EventLog.Information);
			}
			catch (Exception e) {
				PostEventLog(e.Message, e.ToString(), EventLog.Error);					
			}
		}
		private void OnRequest(object source, RequestEventArgs args)
		{
			IHttpClientContext context = (IHttpClientContext)source;
			IHttpRequest request = args.Request;
			NameValueCollection queryList;
			string body = "result=0";
			long length;

			try {
				if (request.BodyIsComplete && request.Method.ToUpper().Trim() == "POST") {
					queryList = HttpUtility.ParseQueryString(GetHttpPostQueryString(request), Encoding.UTF8);
				}
				else {
					queryList = HttpUtility.ParseQueryString(request.Uri.Query, Encoding.UTF8);
				}
				body = GetHttpCallbackResponse(queryList);
				length = Encoding.UTF8.GetBytes(body).Length;
				string response = string.IsNullOrEmpty(body)
								? "HTTP/1.1" + " " + (int) HttpStatusCode.OK + " " + "" + "\r\n\r\n"
								: string.Format("{0} {1} {2}\r\nContent-Type: {5}\r\nContent-Length: {3}\r\n\r\n{4}",
											"HTTP/1.1" , (int) HttpStatusCode.OK , "" ?? HttpStatusCode.OK.ToString(),
											length, body, "text/plain");
				byte[] buffer = Encoding.UTF8.GetBytes(response);
				context.Send(buffer);
			}
			catch (Exception e) {
				PostEventLog(e.Message, e.ToString(), EventLog.Error);					
			}
			finally {
				try {
					context.Close();
				}
				catch (Exception e) {
					PostEventLog(e.Message, e.ToString(), EventLog.Error);					
				}
			}
		}
		private string GetHttpPostQueryString(IHttpRequest httpListenerRequest)
		{
			string returnValue = "";
			StreamReader reader;

			try {
				reader = new StreamReader(httpListenerRequest.Body, Encoding.UTF8);
				returnValue = reader.ReadToEnd();
			}
			catch (Exception e) {
				PostEventLog(e.Message, e.ToString(), EventLog.Error);					
			}
			return returnValue;
		}
		private string GetQueryParameter(NameValueCollection queryList, string parameterName) 
		{
			string[] parameterList = queryList.GetValues(parameterName);
			if ((parameterList != null) && (parameterList.Length == 1)) {
				return parameterList[0];
			}
			return "";
		}
		#endregion
		#region Connector Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
		}
		private void PostSendMessage(string toAddress, string fromAddress, string messageText,
		                             string messageType, string gateway, string userId, string userInfo, int priority)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOut");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("UserId", userId);
			messagePacket.Add("UserInfo", userInfo);
			messagePacket.Add("Priority", priority.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		#endregion
	}
}
