// Diafaan SMS Server Scripting Connector skeleton script
// AddReference System.dll
// AddReference Microsoft.VisualBasic.dll
//
using System;
using System.Collections;
using System.Timers;
using System.Collections.Generic;
using System.IO;
using Microsoft.VisualBasic.FileIO;

namespace DiafaanMessageServer
{
	public class ConnectorScript : IScript
	{
		private IScriptHost host = null;
		private Timer filePollTimer = null;
		private string csvFileName = @"c:\temp\messages.csv";
		private int filePollInverval = 5;
  
		public void OnLoad(IScriptHost host)
		{
			this.host = host;
			filePollTimer = new Timer(filePollInverval * 1000);
			filePollTimer.Elapsed += new ElapsedEventHandler(FilePollEvent);
			filePollTimer.Enabled = true;
		}

		public void OnUnload()
		{
			// Important, the timer must be disabled before reloading the script
			filePollTimer.Enabled = false;
			filePollTimer.Elapsed -= FilePollEvent;
			filePollTimer = null;
		}
		// *******************************************************************************************
		// * The FilePollEvent function reads the .CSV file, removes the file and sends the messages *
		// *******************************************************************************************
    private void FilePollEvent(object source, ElapsedEventArgs e)
    {
    	List<KeyValuePair<string, string>> messageList = new List<KeyValuePair<string, string>>();
    	
    	try {
	    	if (File.Exists(csvFileName)) {
	 				using (TextFieldParser parser = new TextFieldParser(csvFileName)) {
						parser.TextFieldType = FieldType.Delimited;
						parser.SetDelimiters(",");
						while (!parser.EndOfData) {
							//Processing row
							string[] fields = parser.ReadFields();
							//Processing the first two fields in the row
							if (fields.Length >= 2) {
								messageList.Add(new KeyValuePair<string, string>(fields[0], fields[1]));
							}
						}
					}
					// Remove the file
					File.Delete(csvFileName);
					// If the file is removed (no exceptions!) the messages can be sent
					foreach (KeyValuePair<string, string> message in messageList) {
						PostSendMessage(message.Key, "", message.Value, "", "", "", "", 0);
					}
				}
			}
			catch (Exception ex) {
				PostEventLog(ex.Message, ex.ToString(), EventLog.Error);
			}
		}
		private void OnMessageReceived(string fromAddress, string toAddress, string message, string messageType,
		                               string messageId, string smsc, string pdu, string gateway,
		                               DateTime sendTime, DateTime receiveTime)
		{
			//
			// TODO: Add code to handle received SMS messages
			// e.g.
			// PostSendMessage(fromAddress, "", "We have received your message", "sms.text", "", "", "", 0);
			//
		}
		private void OnSendMessageResult(string toAddress, string fromAddress, string message, string messageType,
		                               	 string messageId, string pdu, string gateway, int statusCode,
		                               	 string statusText, string errorCode, string errorText,
		                               	 DateTime sendTime, DateTime receiveTime, string userId, string userInfo)
		{
			//
			// TODO: Add code to handle the status result of a previously sent message
			//
		}
		private void OnSendMessageResultUpdate(string messageId, int statusCode, string statusText, 
		                                       string errorCode, string errorText, DateTime receiveTime)
		{
			//
			// TODO: Add code to handle a status result update of a previously sent message
			//
		}
		#region Connector Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
			if (GetPacketString(messagePacket, "PacketName", "") == "MessageIn") {
				OnMessageReceived(GetPacketString(messagePacket, "From", ""),
				                	GetPacketString(messagePacket, "To", ""),
				                	GetPacketString(messagePacket, "Message", ""),
				                	GetPacketString(messagePacket, "MessageType", ""),
				                	GetPacketString(messagePacket, "MessageId", ""),
				                	GetPacketString(messagePacket, "SMSC", ""),
				                	GetPacketString(messagePacket, "PDU", ""),
				                	GetPacketString(messagePacket, "Gateway", ""),
				                	GetPacketDateTime(messagePacket, "SendTime", DateTime.Now),
				                	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.Now));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "MessageOutResult") {
				OnSendMessageResult(GetPacketString(messagePacket, "To", ""),
				                    GetPacketString(messagePacket, "From", ""),
					                	GetPacketString(messagePacket, "Message", ""),
					                	GetPacketString(messagePacket, "MessageType", ""),
					                	GetPacketString(messagePacket, "MessageId", ""),
					                	GetPacketString(messagePacket, "PDU", ""),
					                	GetPacketString(messagePacket, "Gateway", ""),
					                	GetPacketInteger(messagePacket, "StatusCode", -1),
					                	GetPacketString(messagePacket, "StatusText", ""),
					                	GetPacketString(messagePacket, "ErrorCode", ""),
					                	GetPacketString(messagePacket, "ErrorText", ""),
					                	GetPacketDateTime(messagePacket, "SendTime", DateTime.Now),
					                	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.MaxValue),
					                	GetPacketString(messagePacket, "UserId", ""),
					                	GetPacketString(messagePacket, "UserInfo", ""));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "MessageOutResultUpdate") {
				OnSendMessageResultUpdate(GetPacketString(messagePacket, "MessageId", ""),
								                	GetPacketInteger(messagePacket, "StatusCode", -1),
								                	GetPacketString(messagePacket, "StatusText", ""),
					  			              	GetPacketString(messagePacket, "ErrorCode", ""),
					        			        	GetPacketString(messagePacket, "ErrorText", ""),
					        			        	GetPacketDateTime(messagePacket, "ReceiveTime", DateTime.MaxValue));
			}
		}
		private void PostSendMessage(string toAddress, string fromAddress, string messageText,
		                             string messageType, string gateway, string userId, string userInfo, int priority)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOut");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("UserId", userId);
			messagePacket.Add("UserInfo", userInfo);
			messagePacket.Add("Priority", priority.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		private string GetPacketString(Hashtable hashtable, string key, string defaultValue)
		{
			try {
				return (string) hashtable[key];
			}
			catch {}
			return defaultValue;
		}

		private int GetPacketInteger(Hashtable hashtable, string key, int defaultValue)
		{
			try {
				return Convert.ToInt32((string) hashtable[key]);
			}
			catch {}
			return defaultValue;
		}

		private DateTime GetPacketDateTime(Hashtable hashtable, string key, DateTime defaultValue)
		{
			try {
				return new DateTime(Convert.ToInt64((string) hashtable[key]));
			}
			catch {}
			return defaultValue;
		}
		#endregion
	}
}
