﻿// Diafaan SMS Server Scripting Connector skeleton script
// AddReference System.dll
// AddReference System.Web.dll
//
using System;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.Web;
using System.Text;
using System.IO;

namespace DiafaanMessageServer
{
	public class BulkSMSGateway : IScript
	{
		private const string userName = "BulkSMS_Username";
		private const string password = "BulkSMS_Password";
		private int maxMessageParts = 3;
		private IScriptHost host = null;
  
		public void OnLoad(IScriptHost host)
		{
			this.host = host;
		}

		public void OnUnload()
		{
		}
		private void OnMessageToSend(string recordId, string toAddress, string fromAddress, 
		                             string message, string messageType)
		{
			HttpWebRequest httpWebRequest;
			HttpWebResponse httpWebResponse;
			byte[] byteData;
			string queryString;
			string responseText;
			string[] responseList;

			try {
				maxMessageParts = 3;
				queryString = "username=" + HttpUtility.UrlEncode(userName) +
											"&password=" + HttpUtility.UrlEncode(password) +
											"&msisdn=" + HttpUtility.UrlEncode(toAddress.TrimStart('+'));
				if ((messageType != "sms.automatic") &&
						(messageType != "sms.text") &&
						(messageType != "sms.unicode") &&
						(messageType != "sms.automatic.flash") &&
						(messageType != "sms.text.flash") &&
						(messageType != "sms.unicode.flash")) {
					messageType = "sms.automatic";
				}
				message = message.Replace("\t", " ").Replace("\r\n", "\n");
				if (messageType.StartsWith("sms.automatic")) {
					messageType = GetAutomaticMessageType(message, messageType);
				}
				if (messageType.StartsWith("sms.unicode")) {
				  // BULKSMS.COM does not support multipart Unicode messages
					maxMessageParts = 1;
					message = message.Substring(0, GetMaxMessageLength(message, maxMessageParts, messageType));
					queryString += "&message=" + StringToHex(message);
					queryString += "&dca=16bit";
				}
				else {
				  message = character_resolve(message);
					message = message.Substring(0, GetMaxMessageLength(message, maxMessageParts, messageType));
					queryString += "&message=" + HttpUtility.UrlEncode(message, System.Text.Encoding.GetEncoding("ISO-8859-1"));
				}
				if (messageType.EndsWith(".flash")) {
					queryString += "&msg_class=0";
				}
				if (maxMessageParts > 1) queryString += "&allow_concat_text_sms=1&concat_text_sms_max_parts=" + maxMessageParts.ToString();
				httpWebRequest = (HttpWebRequest) WebRequest.Create(@"http://bulksms.vsms.net:5567/eapi/submission/send_sms/2/2.0");
				httpWebRequest.ContentType = "application/x-www-form-urlencoded";
				httpWebRequest.Method = "POST";
				byteData = UTF8Encoding.UTF8.GetBytes(queryString);
				httpWebRequest.ContentLength = byteData.Length;  
				using (Stream postStream = httpWebRequest.GetRequestStream()) {  
					postStream.Write(byteData, 0, byteData.Length);  
				}
				using (httpWebResponse = (HttpWebResponse) httpWebRequest.GetResponse()) {
   		    responseText = new StreamReader(httpWebResponse.GetResponseStream()).ReadToEnd();
   		    responseList = responseText.Trim(new char[] {' ','\r','\n'}).Split('|');
   		    if (responseList[0].Trim() == "0") {
   		    	PostSendResult(recordId, responseList[2], StatusCode.Sent, "Success: message sent", "", "", true);
   		    }
   		    else {
   		    	PostSendResult(recordId, "", StatusCode.SendError, "Error: message rejected", responseList[0], responseList[1].ToLower(), false);
   		    }
				}
			}
			catch (Exception e) {
				PostSendResult(recordId, "", StatusCode.SendError, "Error: message rejected", "", e.Message, false);
				PostEventLog(e.Message, e.ToString(), EventLog.Error);
			}
		}
		private void OnStatusRequest(string messageId)
		{
			HttpWebRequest httpWebRequest;
			HttpWebResponse httpWebResponse;
			byte[] byteData;
			string queryString;
			string responseText;
			string[] responseLines;
			string[] responseList;

			try {
				queryString = "username=" + HttpUtility.UrlEncode(userName) +
											"&password=" + HttpUtility.UrlEncode(password) +
											"&batch_id=" + HttpUtility.UrlEncode(messageId);  
				httpWebRequest = (HttpWebRequest) WebRequest.Create(@"http://bulksms.vsms.net:5567/eapi/status_reports/get_report/2/2.0");
				httpWebRequest.ContentType = "application/x-www-form-urlencoded";
				httpWebRequest.Method = "POST";
				byteData = UTF8Encoding.UTF8.GetBytes(queryString);
				httpWebRequest.ContentLength = byteData.Length;  
				using (Stream postStream = httpWebRequest.GetRequestStream()) {  
					postStream.Write(byteData, 0, byteData.Length);  
				}
				using (httpWebResponse = (HttpWebResponse) httpWebRequest.GetResponse()) {
   		    responseText = new StreamReader(httpWebResponse.GetResponseStream()).ReadToEnd();
					responseLines = responseText.Split(new char[] {'\r','\n'}, StringSplitOptions.RemoveEmptyEntries);		    
   		    responseList = responseLines[0].Split('|');
   		    if (responseList[0].Trim() == "0") {
	   		    responseList = responseLines[1].Split('|');
	   		    if (responseList[1].Trim() == "11") {
  	 		    	PostSendResultUpdate(messageId, StatusCode.Received, "Success: Message received on handset", 
			                             "", "", DateTime.Now);
   			    }
   		    }
				}
			}
			catch (Exception e) {
				PostEventLog(e.Message, e.ToString(), EventLog.Error);
			}
		}
		
		#region BulkSMS gateway helper functions
		public static string GsmDefaultCharSet = "@£$¥èéùìòÇ\nØø\rÅåΔ_ΦΓΛΩΠΨΣΘΞ\u001BÆæßÉ !\"#¤%&'()*+,-./" +
																"0123456789:;<=>?¡ABCDEFGHIJKLMNOPQRSTUVWXYZÄÖÑÜ§¿ab" +
																"cdefghijklmnopqrstuvwxyzäöñüà";
		public static string GsmEscapeCharSet  = "          \f         ^                   {}     \\ " +
																"           [~] |                                   " +
																" €                          ";

		private static string StringToHex(string s)
		{
			string hex = "";
			foreach (char c in s)	{
				int tmp = c;
				hex += String.Format("{0:x4}", (uint)System.Convert.ToUInt32(tmp.ToString()));
			}
			return hex;
		}
		private static string GetAutomaticMessageType(string message, string messageType) 
		{
			int i;
			string returnValue = "sms.text";
			bool useFlash = messageType.EndsWith(".flash");
			string gsmStandardCharacterSet = GsmDefaultCharSet + GsmEscapeCharSet.Replace(" ", "");
			
			i = 0;
			while ((returnValue == "sms.text") && (i<message.Length)) {
				if (gsmStandardCharacterSet.IndexOf(message[i]) == -1) returnValue = "sms.unicode";
				i++;
			}
			if (useFlash) returnValue += ".flash";
			return returnValue;
		}
		private static int GetMaxMessageLength(string message, int maxMessageParts, string messageType) 
		{
			int returnValue = message.Length;
			int escapedMessageLenght = message.Length;
			string extendedCharacterSet = GsmEscapeCharSet.Replace(" ", "");
			int messagePartLength;
			int i;
			
			if (messageType.StartsWith("sms.text")) {
				for (i=0; i<message.Length; i++) {
					if (extendedCharacterSet.IndexOf(message[i]) != -1) escapedMessageLenght ++;
				}
				if (escapedMessageLenght > 160) {
					messagePartLength = (maxMessageParts == 1) ? 160 : 153;
					if (escapedMessageLenght > messagePartLength) {
						escapedMessageLenght = 0;
						for (i=0; i<message.Length; i++) {
							escapedMessageLenght++;
							if (extendedCharacterSet.IndexOf(message[i]) != -1) escapedMessageLenght++;
							if (escapedMessageLenght <= maxMessageParts*messagePartLength) {
								returnValue = i+1;
							}
						}
					}						
				}
			}
			else if (messageType.StartsWith("sms.unicode")) {
				if (message.Length > 70) {
					messagePartLength = (maxMessageParts == 1) ? 70 : 67;
					if (message.Length > messagePartLength*maxMessageParts) {
						returnValue = messagePartLength*maxMessageParts;
					}
				}				
			}
			return returnValue;
		}
		public static string character_resolve(string body) {
			Hashtable chrs = new Hashtable();
			// Greek characters
			chrs.Add('\u03A9', "\u00DB");
			chrs.Add('\u0398', "\u00D4");
			chrs.Add('\u0394', "\u00D0");
			chrs.Add('\u03A6', "\u00DE");
			chrs.Add('\u0393', "\u00AC");
			chrs.Add('\u039B', "\u00C2");
			chrs.Add('\u03A0', "\u00BA");
			chrs.Add('\u03A8', "\u00DD");
			chrs.Add('\u03A3', "\u00CA");
			chrs.Add('\u039E', "\u00B1");
			// Extended characters
			chrs.Add('^', "\u00BB\u0014");
			chrs.Add('{', "\u00BB\u0028");
			chrs.Add('}', "\u00BB\u0029");
			chrs.Add('\\', "\u00BB\u002F");
			chrs.Add('[', "\u00BB\u003C");
			chrs.Add('~', "\u00BB\u003D");
			chrs.Add(']', "\u00BB\u003E");
			chrs.Add('|', "\u00BB\u0040");
			chrs.Add('\u20AC', "\u00BB\u0065");

			string ret_str = "";
			foreach ( char c in body ) {
				if( chrs.ContainsKey( c ) ) {
					ret_str += chrs[ c ];
				}
				else {
					ret_str += c;
				}
			}
			return ret_str;
		}

		#endregion
		
		#region Gateway Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
			if (GetPacketString(messagePacket, "PacketName", "") == "MessageOut") {
				OnMessageToSend(GetPacketString(messagePacket, "RecordId", ""),
				                GetPacketString(messagePacket, "To", ""),
				                GetPacketString(messagePacket, "From", ""),
				                GetPacketString(messagePacket, "Message", ""),
				                GetPacketString(messagePacket, "MessageType", ""));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "StatusRequest") {
				OnStatusRequest(GetPacketString(messagePacket, "MessageId", ""));
			}
		}
		private void PostReceiveMessage(string toAddress, string fromAddress, string messageText,
		                                string messageType, string messageId, DateTime sendTime,
		                                DateTime receiveTime)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageIn");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("SendTime", sendTime.Ticks.ToString());
			messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostDispatchMessage(string recordId, string toAddress, string fromAddress,
		                            		 string message, string messageType, string gateway,
		                            		 string backupGateway)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "DispatchMessage");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", message);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("BackupGateway", backupGateway);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResult(string recordId, string messageId, StatusCode status,
		                            string statusText, string errorCode, string errorText, bool sendStatusRequest)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResult");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			messagePacket.Add("SendStatusRequest", sendStatusRequest ? "1" : "0");
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResultUpdate(string messageId, StatusCode status, string statusText, 
		                                  string errorCode, string errorText, DateTime receiveTime)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResultUpdate");
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			if (status == StatusCode.Received) messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		private string GetPacketString(Hashtable hashtable, string key, string defaultValue)
		{
			try {
				return (string) hashtable[key];
			}
			catch {}
			return defaultValue;
		}

		private int GetPacketInteger(Hashtable hashtable, string key, int defaultValue)
		{
			try {
				return Convert.ToInt32((string) hashtable[key]);
			}
			catch {}
			return defaultValue;
		}

		private DateTime GetPacketDateTime(Hashtable hashtable, string key, DateTime defaultValue)
		{
			try {
				return new DateTime(Convert.ToInt64((string) hashtable[key]));
			}
			catch {}
			return defaultValue;
		}
		#endregion
	}
}
