// Diafaan SMS Server minimal JSON example script for the Clickatell Platform API
// AddReference System.dll
// AddReference Newtonsoft.Json.dll
//
using System;
using System.Collections;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;
using System.Web;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;



namespace DiafaanMessageServer
{
	public class GatewayScript : IScript
	{
		private IScriptHost host = null;
		
		private string smsServerUri = "https://platform.clickatell.com/messages";		
		private string smsServerAuthentication = ""; // Fill in the Clickatell Platform API Key
  
		public void OnLoad(IScriptHost host)
		{
			this.host = host;
			//
			// TODO: Add initialization code.
			//
		}

		public void OnUnload()
		{
			//
			// TODO: Add cleanup code, make sure to remove (Timer) event handlers here
			//
		}
		private void OnMessageToSend(string recordId, string toAddress, string fromAddress, 
		                             string message, string messageType)
		{
			string jsonResponse = null;
			HttpWebRequest request;
			string jsonRequestString;
			
			string messageId = "";
			StatusCode statusCode = StatusCode.SendError;
			string statusMessage = "Error: Unhandled error";
			string errorCode = "";
			string errorMessage = "";
			bool sendStatusRequest = false;

			// Create the JSON request for the SMS message
			JObject jsonMessage = new JObject();				
			JArray toList = new JArray();
			toList.Add(toAddress);
			jsonMessage.Add("to", toList);
			//jsonMessage.Add("from", fromAddress);
			jsonMessage.Add("content", message);
			jsonRequestString = jsonMessage.ToString(Newtonsoft.Json.Formatting.None);
			
			// Create the HTTP request and send the headers
			PostTraceLog("HTTP POST", TraceLog.Information);
			PostTraceLog(smsServerUri, TraceLog.Send);			
			request = (HttpWebRequest)WebRequest.Create(smsServerUri );
			request.Method = "POST";
			request.Headers.Add("Authorization", smsServerAuthentication );
			request.ContentType = "application/json";
			request.Accept = "application/json";

			// Send the HTTP POST data
			byte[] data = Encoding.UTF8.GetBytes(jsonRequestString);
			request.ContentLength = data.Length;
			using (Stream writeStream = request.GetRequestStream()) 
			{
				PostTraceLog(jsonRequestString, TraceLog.Send);
				writeStream.Write(data, 0, data.Length);
				writeStream.Close();
			}
			
			// Receive the JSON response from the SMS server
			using (HttpWebResponse response = (HttpWebResponse)request.GetResponse()) 
			{
				int httpResponseStatusCode = (int) response.StatusCode;
				Stream responseStream = response.GetResponseStream();
				jsonResponse = new StreamReader(responseStream).ReadToEnd();
				PostTraceLog(jsonResponse, TraceLog.Receive);
			}
			
			// Parse the JSON response and add the message to the send log of Diafaan SMS Server
			try {
				if (jsonResponse != null) {
					JObject doc = JObject.Parse(jsonResponse);
					if (doc != null) {
						bool status = false;
						string id = null;
						try {						
							status = (bool)doc["messages"][0]["accepted"];
							id = (string)doc["messages"][0]["apiMessageId"];
						}
						catch {}
						if ((id != null) && (status == true)) {
							statusCode = StatusCode.Sent;
							statusMessage = "Success: Message accepted";
							if (id != null) messageId = id;
						}
						else {
							try {
								errorMessage = (string)doc["messages"][0]["error"];
							}
							catch {}
							if (errorMessage == "") {
								try {
									errorMessage = (string)doc["error"];
								}
								catch {}
							}
							if (errorMessage == "") {
								statusMessage = "Unexpected or missing JSON response";
							}
							else {
								statusMessage = "Error: Message rejected";
								errorCode = "0";
							}
							PostEventLog(statusMessage, jsonResponse, EventLog.Error);
							PostTraceLog(statusMessage, TraceLog.Receive);
						}
					}
					else {
						statusMessage = "Unexpected or missing JSON response";
						PostEventLog(statusMessage, jsonResponse, EventLog.Error);
						PostTraceLog(statusMessage, TraceLog.Receive);
					}
				}
			}
			catch (Exception e) {
				statusMessage = "Error: Invalid JSON response";
				PostEventLog(e.Message, e.ToString(), EventLog.Error);
				PostTraceLog(e.Message, TraceLog.Receive);
			}

			PostSendResult(recordId, messageId, statusCode, statusMessage,
											errorCode, errorMessage, sendStatusRequest);
		}
		private void OnStatusRequest(string messageId)
		{
			// TODO: Send status request query
		}
		#region Gateway Script helper functions
		private enum StatusCode {Sent=200, Received=201, SendError=300}
		private enum EventLog {Information=0, Warning=1, Error=2}
		private enum TraceLog {Information=0, Send=1, Receive=2}

		public void OnMessagePacket(Hashtable messagePacket)
		{
			if (GetPacketString(messagePacket, "PacketName", "") == "MessageOut") {
				OnMessageToSend(GetPacketString(messagePacket, "RecordId", ""),
				                GetPacketString(messagePacket, "To", ""),
				                GetPacketString(messagePacket, "From", ""),
				                GetPacketString(messagePacket, "Message", ""),
				                GetPacketString(messagePacket, "MessageType", ""));
			}
			else if (GetPacketString(messagePacket, "PacketName", "") == "StatusRequest") {
				OnStatusRequest(GetPacketString(messagePacket, "MessageId", ""));
			}
		}
		private void PostReceiveMessage(string toAddress, string fromAddress, string messageText,
		                                string messageType, string messageId, DateTime sendTime,
		                                DateTime receiveTime, string Pdu, string connectorList)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageIn");
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", messageText);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("SendTime", sendTime.Ticks.ToString());
			messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			messagePacket.Add("PDU", Pdu);
			if (connectorList != "") {
				messagePacket.Add("ConnectorList", connectorList);
			}
			host.PostMessagePacket(messagePacket);			
		}
		private void PostDispatchMessage(string recordId, string toAddress, string fromAddress,
		                            		 string message, string messageType, string gateway,
		                            		 string backupGateway)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "DispatchMessage");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("To", toAddress);
			messagePacket.Add("From", fromAddress);
			messagePacket.Add("Message", message);
			messagePacket.Add("MessageType", messageType);
			messagePacket.Add("Gateway", gateway);
			messagePacket.Add("BackupGateway", backupGateway);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResult(string recordId, string messageId, StatusCode status,
		                            string statusText, string errorCode, string errorText, bool sendStatusRequest)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResult");
			messagePacket.Add("RecordId", recordId);
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			if (sendStatusRequest) {
				messagePacket.Add("SendStatusRequest", "1");
			}
			else {
				messagePacket.Add("SendStatusRequest", "0");
			}
			host.PostMessagePacket(messagePacket);			
		}
		private void PostSendResultUpdate(string messageId, StatusCode status, string statusText, 
		                                  string errorCode, string errorText, DateTime receiveTime)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "MessageOutResultUpdate");
			messagePacket.Add("MessageId", messageId);
			messagePacket.Add("StatusCode", ((int) status).ToString());
			messagePacket.Add("StatusText", statusText);
			messagePacket.Add("ErrorCode", errorCode);
			messagePacket.Add("ErrorText", errorText);
			if (status == StatusCode.Received) messagePacket.Add("ReceiveTime", receiveTime.Ticks.ToString());
			host.PostMessagePacket(messagePacket);			
		}
		private void PostEventLog(string eventMessage, string eventDescription, EventLog eventType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "EventLog");
			messagePacket.Add("EventType", ((int) eventType).ToString());
			messagePacket.Add("EventMessage", eventMessage);
			messagePacket.Add("EventDescription", eventDescription);
			host.PostMessagePacket(messagePacket);			
		}
		private void PostTraceLog(string traceLogMessage, TraceLog logType)
		{
			Hashtable messagePacket = new Hashtable();
			
			messagePacket.Add("PacketName", "TraceLog");
			messagePacket.Add("TraceLogType", ((int) logType).ToString());
			messagePacket.Add("TraceLogMessage", traceLogMessage);
			host.PostMessagePacket(messagePacket);			
		}
		private string GetPacketString(Hashtable hashtable, string key, string defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return (string) hashtable[key];
			}
			catch {}
			return defaultValue;
		}
		private int GetPacketInteger(Hashtable hashtable, string key, int defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return Convert.ToInt32((string) hashtable[key]);
			}
			catch {}
			return defaultValue;
		}
		private DateTime GetPacketDateTime(Hashtable hashtable, string key, DateTime defaultValue)
		{
			try {
				if (!hashtable.ContainsKey(key) || (hashtable[key] == null) || (hashtable[key].GetType() != typeof(string))) return defaultValue;
				return new DateTime(Convert.ToInt64((string) hashtable[key]));
			}
			catch {}
			return defaultValue;
		}
		#endregion
	}
}
